; ##############################################################################
; ##############################################################################
; ##############################################################################
; I2C / TWI MASTER EINSTELLUNGEN												
.equ TWI_BIT_RATE			= 20		; 50	; Bit-Rate						
.equ TWI_PRESCALER			= 1			; 2		; Vorteiler: 0=1 1=4 2=16 3=64	
.equ TWI_SCL_BERECHNET		= TAKT/(16+(2*TWI_BIT_RATE)*(exp2(TWI_PRESCALER))*(exp2(TWI_PRESCALER)))
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_INITIALISIERUNG:
	; SCL(Hz) = CPU(Hz) / (16 + 2 * TWBR * 4^TWPS)	
	; Beispiel:										
	; 100.000 = 16.000.000 / (16 + 2 * 18 * 4^1)	
	
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp
	; PULL-UP ein								
	sbi PORTC,PC4
	sbi PORTC,PC5
	; kleine Pause								
	rcall wait_10ms
ret
;###############################################################################
;###############################################################################
;###############################################################################
TWI_BUS_CHECK:
	sbis PINC,5				; springe, wenn PIN HIGH....
	rjmp TWI_BUS_CHECK_ERROR_SCL
	sbis PINC,4				; springe, wenn PIN HIGH....
	rjmp TWI_BUS_CHECK_ERROR_SDA
	ret
;-------------------------------------------------------------------------------
TWI_BUS_CHECK_ERROR_SCL:
	ldi temp4, (ZEICHEN_S)
	ldi temp3, (ZEICHEN_C)
	ldi temp2, (ZEICHEN_L)
	ldi temp1, (ZEICHEN_AUS)

	STS(adr_LED_BYTE_0),temp1
	STS(adr_LED_BYTE_1),temp2
	STS(adr_LED_BYTE_2),temp3
	STS(adr_LED_BYTE_3),temp4
	rcall wait_100ms
	STS(adr_LED_BYTE_0),NULL
	STS(adr_LED_BYTE_1),NULL
	STS(adr_LED_BYTE_2),NULL
	STS(adr_LED_BYTE_3),NULL
	rcall wait_100ms
	rcall TWI_ERROR_SCHWER
	rjmp TWI_BUS_CHECK
;-------------------------------------------------------------------------------
TWI_BUS_CHECK_ERROR_SDA:
	ldi temp4, (ZEICHEN_S)
	ldi temp3, (ZEICHEN_C)
	ldi temp2, (ZEICHEN_L)
	ldi temp1, (ZEICHEN_AUS)

	STS(adr_LED_BYTE_0),temp1
	STS(adr_LED_BYTE_1),temp2
	STS(adr_LED_BYTE_2),temp3
	STS(adr_LED_BYTE_3),temp4
	rcall wait_100ms
	STS(adr_LED_BYTE_0),NULL
	STS(adr_LED_BYTE_1),NULL
	STS(adr_LED_BYTE_2),NULL
	STS(adr_LED_BYTE_3),NULL
	rcall wait_100ms
	rcall TWI_ERROR_SCHWER
	rjmp TWI_BUS_CHECK
;###############################################################################
;###############################################################################
;###############################################################################
; SCAN nach vorhandenen SLAVES													
TWI_SCAN:
	ldi temp4, (ZEICHEN_S)
	ldi temp3, (ZEICHEN_C)
	ldi temp2, (ZEICHEN_A)
	ldi temp1, (ZEICHEN_N)

	STS(adr_LED_BYTE_0),temp1
	STS(adr_LED_BYTE_1),temp2
	STS(adr_LED_BYTE_2),temp3
	STS(adr_LED_BYTE_3),temp4
	rcall wait_100ms

	ldi temp9,0
TWI_SCAN_s:
	mov temp1,temp9
	rcall TWI_MASTER_TRANSMITTER_START_TEMP1
	tst TWI_FEHLER_CODE
	brne TWI_SCAN_w
	push temp9
    mov temp1,temp9
	clr temp2
	rcall ANZEIGE_ZAHL_3_STELLEN_TEMP1
	rcall wait_100ms
	pop temp9
TWI_SCAN_w:
	inc temp9
	cpi temp9,127
	brne TWI_SCAN_s
	rcall TWI_STOP
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; START-ROUTINE	(MASTER TRANSMITTER)										   	
;																			   	
; INPUT:	temp1 <= ADRESSE SLAVE											   	
; OUT: 		TWI_FEHLER_CODE														
;																				
TWI_MASTER_TRANSMITTER_START_TEMP1:
	; INITIALISIERUNG	
	clr TWI_FEHLER_CODE							; FEHLER lschen	
	WDR
	; START-MT			
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1	
	; Check TWSR						
	in temp,TWSR													
	andi temp, 0xF8
	cpi temp,  0x08								; 0x08 START erfolgreich
	breq TWI_MASTER_TRANSMITTER_START_w
	cpi temp,  0x10								; 0x10 (repeated START)
	breq TWI_MASTER_TRANSMITTER_START_w
	; ERROR								
	rjmp TWI_MASTER_TRANSMITTER_START_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_w:
	; Load SLA + "W"  (W=0) 			
	LSL  temp1									; Adresse 1 x links	
	out TWDR, temp1
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1								
	; Check TWSR						
	in temp,TWSR												
	andi temp, 0xF8
	cpi temp,  0x18								; 0x18				
	brne TWI_MASTER_TRANSMITTER_START_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_START_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   	
; Ein BYTE zum SLAVE zu senden										   	   	   	
; 																			   	
; INPUT:	temp1 <= DATEN													   	
; OUT: 		TWI_FEHLER_CODE														
; 																			   	
TWI_MASTER_TRANSMITTER_DATEN_TEMP1:
	WDR
	; ERROR								
	tst TWI_FEHLER_CODE						; Fehler =0 ?	
	breq TWI_MASTER_TRANSMITTER_DATEN_RUN	; ja			
	ret										; zurck		
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_RUN:
	out TWDR, temp1							; 	Load DATA	
	; senden - Clear TWINT 				
	ldi temp, (1<<TWINT) | (1<<TWEN)							
	out TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	; Check TWSR						
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x28	; 0x28
	brne TWI_MASTER_TRANSMITTER_DATEN_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_TRANSMITTER_DATEN_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			   	
; TWI/I2C START-ROUTINE														   	
;																			   	
; INPUT:	temp1 <= ADRESSE SLAVE											   	
; OUT: 		TWI_FEHLER_CODE														
;																			   	
TWI_MASTER_RECEIVER_START_TEMP1:
	; INITIALISIERUNG	
	clr TWI_FEHLER_CODE
	WDR
	; START-MR 											
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN )
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1						
	; Check TWSR 										
	in temp,TWSR		
	andi temp, 0xF8
	cpi temp,  0x08 							; 0x08 START erfolgreich
	breq TWI_MASTER_RECEIVER_START_RUN
	cpi temp,  0x10								; 0x10 (repeated START)	
	breq TWI_MASTER_RECEIVER_START_RUN
	rjmp TWI_MASTER_RECEIVER_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_START_RUN:
	; Load SLA +"R" (R=1)								
	LSL  temp1					; 1 x links	
	ori temp1, 0b00000001 		; R=1		
	out TWDR, temp1
	; senden - Clear TWINT 								
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait		; warten, bis TWINT wieder = 1						
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x40 				;SLA+R gesendet; ACK empfangen
	brne TWI_MASTER_RECEIVER_ERROR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_ERROR:
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTES empfangen werden sollen										
;																				
; OUT: 		temp1																
; OUT: 		TWI_FEHLER_CODE														
;																				
TWI_MASTER_RECEIVER_BYTE_TEMP1:
	WDR
	; ERROR								
	tst TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_RUN:
	; senden "ACK" (TWEA=1) fr mehrere Daten-Bytes 	
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait; TWINT abfragen									
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi temp, 0x50	; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_BYTE_ERROR
	; DATEN einlesen							
	in temp1, TWDR						
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_BYTE_ERROR:
	; ERROR								
	clr temp1
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; Achtung, diese Routine verwenden, wenn 										
; nur ein! BYTE empfangen werden soll, oder wenn								
; das letzte! BYTE empfangen werden soll										
;																				
; OUT: temp1																	
; OUT:TWI_FEHLER_CODE															
;																				
TWI_MASTER_RECEIVER_LAST_BYTE_TEMP1:
	; ERROR								
	tst TWI_FEHLER_CODE
	breq TWI_MASTER_RECEIVER_LAST_BYTE_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_RUN:
	; senden "NOT ACK" (TWEA=0) fr letztes bzw fr ein Daten-Byte
	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )
	out TWCR, temp
	rcall TWI_TWINT_wait	; TWINT abfragen									
	; Check TWSR										
	in temp,TWSR
	andi temp, 0xF8
	cpi  temp, 0x58	; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RECEIVER_LAST_BYTE_ERROR
	; DATEN einlesen		
	in temp1, TWDR
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RECEIVER_LAST_BYTE_ERROR:
	; ERROR								
	clr temp1
	mov TWI_FEHLER_CODE, EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; STOP- Routine																   
; 																			   
TWI_STOP:
	; kleine Pause
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE	; STOP								
	out TWCR, temp	
	; kleine Pause
	rcall wait_10ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_AUS:
	ldi temp, 1<<TWINT|0<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|0<<TWEN|0<<TWIE
	out TWCR,temp
	; kleine Pause
	rcall wait_10ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																			   
; warten, bis das TWINT Flag gesetzt ist									   
; 																			   
TWI_TWINT_wait:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_wait	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_ERROR_SCHWER:
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_STOP
	rcall TWI_AUS
	rcall TWI_INITIALISIERUNG
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
